@@ -10,6 +10,7 @@
 #include <stdlib.h>
 #include <linux/if_infiniband.h>
 
+#include "libnm-platform/nmp-base.h"
 #include "nm-utils.h"
 #include "nm-utils-private.h"
 #include "nm-setting-private.h"
@@ -36,7 +37,9 @@ typedef struct {
     char   *transport_mode;
     char   *parent;
     char   *virtual_iface_name;
-    int     p_key;
+    gsize   virtual_iface_name_parent_length;
+    gint32  virtual_iface_name_p_key;
+    gint32  p_key;
     guint32 mtu;
 } NMSettingInfinibandPrivate;
 
@@ -142,6 +145,17 @@ nm_setting_infiniband_get_parent(NMSettingInfiniband *setting)
     return NM_SETTING_INFINIBAND_GET_PRIVATE(setting)->parent;
 }
 
+char *
+nm_setting_infiniband_create_virtual_interface_name(const char *parent, int p_key)
+{
+    char *s;
+
+    s = g_strdup_printf("%s.%04x", parent, (guint) p_key);
+    if (strlen(s) >= NMP_IFNAMSIZ)
+        s[NMP_IFNAMSIZ - 1] = '\0';
+    return s;
+}
+
 /**
  * nm_setting_infiniband_get_virtual_interface_name:
  * @setting: the #NMSettingInfiniband
@@ -156,21 +170,32 @@ const char *
 nm_setting_infiniband_get_virtual_interface_name(NMSettingInfiniband *setting)
 {
     NMSettingInfinibandPrivate *priv = NM_SETTING_INFINIBAND_GET_PRIVATE(setting);
+    gsize                       len;
 
-    if (priv->p_key == -1 || !priv->parent)
+    if (priv->p_key == -1 || !priv->parent) {
+        nm_clear_g_free(&priv->virtual_iface_name);
         return NULL;
+    }
 
-    if (!priv->virtual_iface_name)
-        priv->virtual_iface_name = g_strdup_printf("%s.%04x", priv->parent, priv->p_key);
+    len = strlen(priv->parent);
+    if (!priv->virtual_iface_name || priv->virtual_iface_name_p_key != priv->p_key
+        || priv->virtual_iface_name_parent_length != len
+        || memcmp(priv->parent, priv->virtual_iface_name, len) != 0) {
+        priv->virtual_iface_name_p_key         = priv->p_key;
+        priv->virtual_iface_name_parent_length = len;
+        g_free(priv->virtual_iface_name);
+        priv->virtual_iface_name =
+            nm_setting_infiniband_create_virtual_interface_name(priv->parent, priv->p_key);
+    }
 
-    return NM_SETTING_INFINIBAND_GET_PRIVATE(setting)->virtual_iface_name;
+    return priv->virtual_iface_name;
 }
 
 static gboolean
 verify(NMSetting *setting, NMConnection *connection, GError **error)
 {
-    NMSettingConnection        *s_con = NULL;
-    NMSettingInfinibandPrivate *priv  = NM_SETTING_INFINIBAND_GET_PRIVATE(setting);
+    NMSettingConnection        *s_con;
+    NMSettingInfinibandPrivate *priv = NM_SETTING_INFINIBAND_GET_PRIVATE(setting);
 
     if (priv->mac_address && !nm_utils_hwaddr_valid(priv->mac_address, INFINIBAND_ALEN)) {
         g_set_error_literal(error,
@@ -229,39 +254,59 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
             g_prefix_error(error, "%s: ", NM_SETTING_INFINIBAND_PARENT);
             return FALSE;
         }
-    }
-
-    if (connection)
-        s_con = nm_connection_get_setting_connection(connection);
-    if (s_con) {
-        const char *interface_name = nm_setting_connection_get_interface_name(s_con);
-
-        if (interface_name && priv->p_key != -1) {
-            if (!priv->virtual_iface_name)
-                priv->virtual_iface_name = g_strdup_printf("%s.%04x", priv->parent, priv->p_key);
-
-            if (strcmp(interface_name, priv->virtual_iface_name) != 0) {
-                /* We don't support renaming software infiniband devices. Later we might, but
-                 * for now just reject such connections.
-                 **/
-                g_set_error(error,
-                            NM_CONNECTION_ERROR,
-                            NM_CONNECTION_ERROR_INVALID_PROPERTY,
-                            _("interface name of software infiniband device must be '%s' or unset "
-                              "(instead it is '%s')"),
-                            priv->virtual_iface_name,
-                            interface_name);
-                g_prefix_error(error,
-                               "%s.%s: ",
-                               NM_SETTING_CONNECTION_SETTING_NAME,
-                               NM_SETTING_CONNECTION_INTERFACE_NAME);
-                return FALSE;
-            }
+        if (NM_IN_SET(priv->p_key, 0, 0x8000)) {
+            g_set_error_literal(error,
+                                NM_CONNECTION_ERROR,
+                                NM_CONNECTION_ERROR_INVALID_PROPERTY,
+                                _("the values 0 and 0x8000 are not allowed"));
+            g_prefix_error(error, "%s: ", NM_SETTING_INFINIBAND_P_KEY);
+            return FALSE;
         }
     }
 
     /* *** errors above here should be always fatal, below NORMALIZABLE_ERROR *** */
 
+    s_con = connection ? nm_connection_get_setting_connection(connection) : NULL;
+
+    if (s_con) {
+        const char *interface_name = nm_setting_connection_get_interface_name(s_con);
+
+        if (interface_name && priv->p_key != -1) {
+            const char *virtual_iface_name;
+
+            virtual_iface_name =
+                nm_setting_infiniband_get_virtual_interface_name(NM_SETTING_INFINIBAND(setting));
+
+            if (!nm_streq0(interface_name, virtual_iface_name)) {
+                /* We don't support renaming software infiniband devices. Later we might, but
+                 * for now just reject such connections.
+                 **/
+                if (virtual_iface_name) {
+                    g_set_error(
+                        error,
+                        NM_CONNECTION_ERROR,
+                        NM_CONNECTION_ERROR_INVALID_PROPERTY,
+                        _("interface name of software infiniband device must be '%s' or unset "
+                          "(instead it is '%s')"),
+                        virtual_iface_name,
+                        interface_name);
+                } else {
+                    g_set_error(error,
+                                NM_CONNECTION_ERROR,
+                                NM_CONNECTION_ERROR_INVALID_PROPERTY,
+                                _("interface name of software infiniband device with MAC address "
+                                  "must be unset (instead it is '%s')"),
+                                interface_name);
+                }
+                g_prefix_error(error,
+                               "%s.%s: ",
+                               NM_SETTING_CONNECTION_SETTING_NAME,
+                               NM_SETTING_CONNECTION_INTERFACE_NAME);
+                return NM_SETTING_VERIFY_NORMALIZABLE_ERROR;
+            }
+        }
+    }
+
     if (priv->mtu > NM_INFINIBAND_MAX_MTU) {
         /* Traditionally, MTU for "datagram" mode was limited to 2044
          * and for "connected" mode it was 65520.
@@ -290,75 +335,9 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
 
 /*****************************************************************************/
 
-static void
-get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
-{
-    NMSettingInfiniband *setting = NM_SETTING_INFINIBAND(object);
-
-    switch (prop_id) {
-    case PROP_MAC_ADDRESS:
-        g_value_set_string(value, nm_setting_infiniband_get_mac_address(setting));
-        break;
-    case PROP_MTU:
-        g_value_set_uint(value, nm_setting_infiniband_get_mtu(setting));
-        break;
-    case PROP_TRANSPORT_MODE:
-        g_value_set_string(value, nm_setting_infiniband_get_transport_mode(setting));
-        break;
-    case PROP_P_KEY:
-        g_value_set_int(value, nm_setting_infiniband_get_p_key(setting));
-        break;
-    case PROP_PARENT:
-        g_value_set_string(value, nm_setting_infiniband_get_parent(setting));
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-        break;
-    }
-}
-
-static void
-set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
-{
-    NMSettingInfinibandPrivate *priv = NM_SETTING_INFINIBAND_GET_PRIVATE(object);
-
-    switch (prop_id) {
-    case PROP_MAC_ADDRESS:
-        g_free(priv->mac_address);
-        priv->mac_address =
-            _nm_utils_hwaddr_canonical_or_invalid(g_value_get_string(value), INFINIBAND_ALEN);
-        break;
-    case PROP_MTU:
-        priv->mtu = g_value_get_uint(value);
-        break;
-    case PROP_TRANSPORT_MODE:
-        g_free(priv->transport_mode);
-        priv->transport_mode = g_value_dup_string(value);
-        break;
-    case PROP_P_KEY:
-        priv->p_key = g_value_get_int(value);
-        nm_clear_g_free(&priv->virtual_iface_name);
-        break;
-    case PROP_PARENT:
-        g_free(priv->parent);
-        priv->parent = g_value_dup_string(value);
-        nm_clear_g_free(&priv->virtual_iface_name);
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-        break;
-    }
-}
-
-/*****************************************************************************/
-
 static void
 nm_setting_infiniband_init(NMSettingInfiniband *self)
-{
-    NMSettingInfinibandPrivate *priv = NM_SETTING_INFINIBAND_GET_PRIVATE(self);
-
-    priv->p_key = -1;
-}
+{}
 
 /**
  * nm_setting_infiniband_new:
@@ -378,8 +357,6 @@ finalize(GObject *object)
 {
     NMSettingInfinibandPrivate *priv = NM_SETTING_INFINIBAND_GET_PRIVATE(object);
 
-    g_free(priv->transport_mode);
-    g_free(priv->parent);
     g_free(priv->virtual_iface_name);
 
     G_OBJECT_CLASS(nm_setting_infiniband_parent_class)->finalize(object);
@@ -394,8 +371,8 @@ nm_setting_infiniband_class_init(NMSettingInfinibandClass *klass)
 
     g_type_class_add_private(klass, sizeof(NMSettingInfinibandPrivate));
 
-    object_class->get_property = get_property;
-    object_class->set_property = set_property;
+    object_class->get_property = _nm_setting_property_get_property_direct;
+    object_class->set_property = _nm_setting_property_set_property_direct;
     object_class->finalize     = finalize;
 
     setting_class->verify = verify;
@@ -414,7 +391,8 @@ nm_setting_infiniband_class_init(NMSettingInfinibandClass *klass)
      *   or semicolon separated list of 20 decimal bytes (obsolete)
      * example: mac-address= 80:00:00:6d:fe:80:00:00:00:00:00:00:00:02:55:00:70:33:cf:01
      * ---end---
-     * ---ifcfg-rh---
+     */
+    /* ---ifcfg-rh---
      * property: mac-address
      * variable: HWADDR
      * description: IBoIP 20-byte hardware address of the device (in traditional
@@ -447,14 +425,16 @@ nm_setting_infiniband_class_init(NMSettingInfinibandClass *klass)
      * description: MTU of the interface.
      * ---end---
      */
-    obj_properties[PROP_MTU] = g_param_spec_uint(NM_SETTING_INFINIBAND_MTU,
-                                                 "",
-                                                 "",
-                                                 0,
-                                                 G_MAXUINT32,
-                                                 0,
-                                                 G_PARAM_READWRITE | NM_SETTING_PARAM_FUZZY_IGNORE
-                                                     | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_uint32(properties_override,
+                                              obj_properties,
+                                              NM_SETTING_INFINIBAND_MTU,
+                                              PROP_MTU,
+                                              0,
+                                              G_MAXUINT32,
+                                              0,
+                                              NM_SETTING_PARAM_FUZZY_IGNORE,
+                                              NMSettingInfinibandPrivate,
+                                              mtu);
 
     /**
      * NMSettingInfiniband:transport-mode:
@@ -470,39 +450,55 @@ nm_setting_infiniband_class_init(NMSettingInfinibandClass *klass)
      *   "datagram" mode
      * ---end---
      */
-    obj_properties[PROP_TRANSPORT_MODE] = g_param_spec_string(
-        NM_SETTING_INFINIBAND_TRANSPORT_MODE,
-        "",
-        "",
-        NULL,
-        G_PARAM_READWRITE | NM_SETTING_PARAM_INFERRABLE | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_string(properties_override,
+                                              obj_properties,
+                                              NM_SETTING_INFINIBAND_TRANSPORT_MODE,
+                                              PROP_TRANSPORT_MODE,
+                                              NM_SETTING_PARAM_INFERRABLE,
+                                              NMSettingInfinibandPrivate,
+                                              transport_mode);
 
     /**
      * NMSettingInfiniband:p-key:
      *
-     * The InfiniBand P_Key to use for this device. A value of -1 means to use
-     * the default P_Key (aka "the P_Key at index 0"). Otherwise, it is a 16-bit
-     * unsigned integer, whose high bit is set if it is a "full membership"
-     * P_Key.
+     * The InfiniBand p-key to use for this device. A value of -1 means to use
+     * the default p-key (aka "the p-key at index 0"). Otherwise, it is a
+     * 16-bit unsigned integer, whose high bit 0x8000 is set if it is a "full
+     * membership" p-key. The values 0 and 0x8000 are not allowed.
+     *
+     * With the p-key set, the interface name is always "$parent.$p_key".
+     * Setting "connection.interface-name" to another name is not supported.
+     *
+     * Note that kernel will internally always set the full membership bit,
+     * although the interface name does not reflect that. Usually the user
+     * would want to configure a full membership p-key with 0x8000 flag set.
      **/
     /* ---ifcfg-rh---
      * property: p-key
-     * variable: PKEY_ID (and PKEY=yes)
+     * variable: PKEY_ID or PKEY_ID_NM(*) (requires PKEY=yes)
      * default: PKEY=no
      * description: InfiniBand P_Key. The value can be a hex number prefixed with "0x"
      *   or a decimal number.
-     *   When PKEY_ID is specified, PHYSDEV and DEVICE also must be specified.
+     *   When PKEY_ID is specified, PHYSDEV must be specified.
+     *   Note that ifcfg-rh format will always automatically set the full membership
+     *   flag 0x8000 for the PKEY_ID variable. To express IDs without the full membership
+     *   flag, use PKEY_ID_NM. Note that kernel internally treats the interface as
+     *   having the full membership flag set, this mainly affects the interface name.
+     *   For the ifcfg file to be supported by initscripts' ifup-ib, the DEVICE=
+     *   must always be set. NetworkManager does not require that.
      * example: PKEY=yes PKEY_ID=2 PHYSDEV=mlx4_ib0 DEVICE=mlx4_ib0.8002
      * ---end---
      */
-    obj_properties[PROP_P_KEY] =
-        g_param_spec_int(NM_SETTING_INFINIBAND_P_KEY,
-                         "",
-                         "",
-                         -1,
-                         0xFFFF,
-                         -1,
-                         G_PARAM_READWRITE | NM_SETTING_PARAM_INFERRABLE | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_int32(properties_override,
+                                             obj_properties,
+                                             NM_SETTING_INFINIBAND_P_KEY,
+                                             PROP_P_KEY,
+                                             -1,
+                                             0xFFFF,
+                                             -1,
+                                             NM_SETTING_PARAM_INFERRABLE,
+                                             NMSettingInfinibandPrivate,
+                                             p_key);
 
     /**
      * NMSettingInfiniband:parent:
@@ -520,12 +516,13 @@ nm_setting_infiniband_class_init(NMSettingInfinibandClass *klass)
      * example: PHYSDEV=ib0
      * ---end---
      */
-    obj_properties[PROP_PARENT] = g_param_spec_string(
-        NM_SETTING_INFINIBAND_PARENT,
-        "",
-        "",
-        NULL,
-        G_PARAM_READWRITE | NM_SETTING_PARAM_INFERRABLE | G_PARAM_STATIC_STRINGS);
+    _nm_setting_property_define_direct_string(properties_override,
+                                              obj_properties,
+                                              NM_SETTING_INFINIBAND_PARENT,
+                                              PROP_PARENT,
+                                              NM_SETTING_PARAM_INFERRABLE,
+                                              NMSettingInfinibandPrivate,
+                                              parent);
 
     g_object_class_install_properties(object_class, _PROPERTY_ENUMS_LAST, obj_properties);
 
