@@ -30,6 +30,7 @@ NM_GOBJECT_PROPERTIES_DEFINE(NMSettingVlan,
                              PROP_PARENT,
                              PROP_ID,
                              PROP_FLAGS,
+                             PROP_PROTOCOL,
                              PROP_INGRESS_PRIORITY_MAP,
                              PROP_EGRESS_PRIORITY_MAP, );
 
@@ -37,6 +38,7 @@ typedef struct {
     GSList *ingress_priority_map;
     GSList *egress_priority_map;
     char   *parent;
+    char   *protocol;
     guint32 id;
     guint32 flags;
 } NMSettingVlanPrivate;
@@ -81,6 +83,22 @@ nm_setting_vlan_get_parent(NMSettingVlan *setting)
     return NM_SETTING_VLAN_GET_PRIVATE(setting)->parent;
 }
 
+/**
+ * nm_setting_vlan_get_protocol:
+ * @setting: the #NMSettingVlan
+ *
+ * Since: 1.42
+ *
+ * Returns: the #NMSettingVlan:protocol property of the setting
+ **/
+const char *
+nm_setting_vlan_get_protocol(NMSettingVlan *setting)
+{
+    g_return_val_if_fail(NM_IS_SETTING_VLAN(setting), NULL);
+
+    return NM_SETTING_VLAN_GET_PRIVATE(setting)->protocol;
+}
+
 /**
  * nm_setting_vlan_get_id:
  * @setting: the #NMSettingVlan
@@ -662,6 +680,16 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
         return FALSE;
     }
 
+    if (!NM_IN_STRSET(priv->protocol, NULL, "802.1Q", "802.1ad")) {
+        g_set_error(error,
+                    NM_CONNECTION_ERROR,
+                    NM_CONNECTION_ERROR_INVALID_PROPERTY,
+                    _("invalid VLAN protocol %s: must be '802.1Q' or '802.1ad'"),
+                    priv->protocol);
+        g_prefix_error(error, "%s.%s: ", NM_SETTING_VLAN_SETTING_NAME, NM_SETTING_VLAN_PROTOCOL);
+        return FALSE;
+    }
+
     if (connection && !s_wired) {
         /* technically, a VLAN setting does not require an ethernet setting. However,
          * the ifcfg-rh reader always adds a ethernet setting when reading a vlan setting.
@@ -915,6 +943,32 @@ nm_setting_vlan_class_init(NMSettingVlanClass *klass)
                                        .from_dbus_fcn = _nm_setting_property_from_dbus_fcn_gprop,
                                        .from_dbus_is_full = TRUE));
 
+    /**
+     * NMSettingVlan:protocol:
+     *
+     * Specifies the VLAN protocol to use for encapsulation.
+     *
+     * Supported values are: '802.1Q', '802.1ad'. If not specified the default
+     * value is '802.1Q'.
+     *
+     * Since: 1.42
+     **/
+    /* ---ifcfg-rh---
+     * property: protocol
+     * variable: VLAN_PROTOCOL
+     * description: VLAN protocol.
+     * example: VLAN_PROTOCOL="802.1ad"
+     * ---end---
+     */
+    _nm_setting_property_define_direct_string(properties_override,
+                                              obj_properties,
+                                              NM_SETTING_VLAN_PROTOCOL,
+                                              PROP_PROTOCOL,
+                                              NM_SETTING_PARAM_INFERRABLE,
+                                              NMSettingVlanPrivate,
+                                              protocol,
+                                              .direct_string_is_refstr = TRUE);
+
     /**
      * NMSettingVlan:ingress-priority-map:
      *
