@@ -322,6 +322,8 @@ typedef gpointer (*NMUtilsCopyFunc)(gpointer);
 const char **
 _nm_ip_address_get_attribute_names(const NMIPAddress *addr, gboolean sorted, guint *out_length);
 
+char *nm_setting_infiniband_create_virtual_interface_name(const char *parent, int p_key);
+
 #define NM_SETTING_WIRED_S390_OPTION_MAX_LEN 200u
 
 void     _nm_setting_wired_clear_s390_options(NMSettingWired *setting);
@@ -395,6 +397,7 @@ GSList *_nm_vpn_plugin_info_list_load_dir(const char               *dirname,
 /*****************************************************************************/
 
 GHashTable *_nm_setting_ovs_external_ids_get_data(NMSettingOvsExternalIDs *self);
+GHashTable *_nm_setting_ovs_other_config_get_data(NMSettingOvsOtherConfig *self);
 
 /*****************************************************************************/
 
