@@ -355,6 +355,18 @@
 #define NM_AVAILABLE_IN_1_42
 #endif
 
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_42_2
+#define NM_AVAILABLE_IN_1_42_2 G_UNAVAILABLE(1, 42.2)
+#else
+#define NM_AVAILABLE_IN_1_42_2
+#endif
+
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_42_8
+#define NM_AVAILABLE_IN_1_42_8 G_UNAVAILABLE(1, 42.8)
+#else
+#define NM_AVAILABLE_IN_1_42_8
+#endif
+
 /*
  * Synchronous API for calling D-Bus in libnm is deprecated. See
  * https://networkmanager.dev/docs/libnm/latest/usage.html#sync-api
